
from django.apps import apps
from django.contrib import admin
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _

from modeltranslation.admin import TranslationAdmin

from articles import config
from articles.models import Article


def get_article_list_display():

    list_display = ['title']

    if config.IS_ARTICLE_TYPE_ENABLED:
        list_display += ['type']

    list_display += ['created']

    if config.ARE_COMMENTS_ENABLED:
        list_display += ['are_comments_enabled']

    return list_display + ['preview_tag']


def get_article_list_filter():

    list_filter = []

    if config.IS_ARTICLE_TYPE_ENABLED:
        list_filter += ['type']

    list_filter += ['created']

    if config.ARE_COMMENTS_ENABLED:
        list_filter += ['are_comments_enabled']

    return list_filter


@admin.register(Article)
class ArticleAdmin(TranslationAdmin):

    list_display = get_article_list_display()

    list_filter = get_article_list_filter()

    if config.ARE_TAGS_ENABLED:
        filter_horizontal = ['tags']

    def preview_tag(self, obj):
        return render_to_string('articles/admin/preview.html', {
            'file': obj.logo
        })

    preview_tag.short_description = _('Preview')


if config.IS_ARTICLE_TYPE_ENABLED:

    @admin.register(apps.get_model('articles', 'ArticleType'))
    class ArticleTypeAdmin(TranslationAdmin):

        list_display = ['name']
        search_fields = ['name']


if config.ARE_TAGS_ENABLED:

    @admin.register(apps.get_model('articles', 'ArticleTag'))
    class ArticleTagAdmin(TranslationAdmin):

        list_display = ['text']
        search_fields = ['text']




from articles import defaults


class DefaultArticleSettings(object):

    IS_ARTICLE_TYPE_ENABLED = defaults.IS_ARTICLE_TYPE_ENABLED
    IS_ARTICLE_HITCOUNT_ENABLED = defaults.IS_ARTICLE_HITCOUNT_ENABLED
    ARE_COMMENTS_ENABLED = defaults.ARE_COMMENTS_ENABLED

    @property
    def INSTALLED_APPS(self):
        apps = super().INSTALLED_APPS

        if self.IS_ARTICLE_HITCOUNT_ENABLED and 'hitcount' not in apps:
            apps += ['hitcount']

        if self.ARE_COMMENTS_ENABLED and 'comments' not in apps:
            apps += ['comments']

        return apps + ['articles']




from django.apps import apps
from django.shortcuts import get_object_or_404
from django.views.generic import ListView, DetailView


from pagination import PaginationMixin

from articles import config
from articles.models import Article


class ArticleListView(PaginationMixin, ListView):

    template_name = 'articles/list.html'
    paginate_by = 10
    queryset = Article.objects.all()

    if config.IS_ARTICLE_TYPE_ENABLED:

        def get(self, request, *args, **kwargs):
            self.article_type = get_object_or_404(
                apps.get_model('articles', 'ArticleType'),
                slug=kwargs['type'])
            return super(ArticleListView, self).get(request, *args, **kwargs)

        def get_queryset(self):
            return self.article_type.articles.all()

        def get_context_data(self, **kwargs):
            cxt = super(ArticleListView, self).get_context_data(**kwargs)
            cxt['article_type'] = self.article_type
            return cxt


def get_detail_view_class():

    if config.IS_ARTICLE_HITCOUNT_ENABLED:
        from hitcount.views import HitCountDetailView
        return HitCountDetailView

    return DetailView


class ArticleDetailView(get_detail_view_class()):

    template_name = 'articles/detail.html'
    count_hit = True

    def get_object(self, queryset=None):

        params = {'pk': self.kwargs['id']}

        if config.IS_ARTICLE_TYPE_ENABLED:
            params['type__slug'] = self.kwargs['type']

        return get_object_or_404(Article, **params)




from django.apps import apps
from django import template


register = template.Library()


@register.simple_tag(takes_context=True)
def get_latest_articles(context, article_type, count=5):
    return apps.get_model('articles', 'Article').objects.filter(
        type__slug=article_type)[:count]


@register.simple_tag(takes_context=True)
def get_most_popular_articles(context, article_type, count=5):
    return apps.get_model('articles', 'Article').most_popular(
        article_type)[:count]




from django.apps import apps

from modeltranslation.translator import translator

from articles import config
from articles.models import Article


translator.register(Article, fields=['title', 'description', 'text'])

if config.ARE_TAGS_ENABLED:
    translator.register(
        apps.get_model('articles', 'ArticleTag'),
        fields=['text'])

if config.IS_ARTICLE_TYPE_ENABLED:
    translator.register(
        apps.get_model('articles', 'ArticleType'),
        fields=['name'])




from django.conf import settings

from articles import defaults


IS_ARTICLE_TYPE_ENABLED = getattr(
    settings, 'IS_ARTICLE_TYPE_ENABLED', defaults.IS_ARTICLE_TYPE_ENABLED)

IS_ARTICLE_HITCOUNT_ENABLED = getattr(
    settings, 'IS_ARTICLE_HITCOUNT_ENABLED', False)

ARE_TAGS_ENABLED = getattr(
    settings, 'ARE_TAGS_ENABLED', False)

ARE_COMMENTS_ENABLED = getattr(
    settings, 'ARE_COMMENTS_ENABLED', False)




from django.urls import path

from articles.config import IS_ARTICLE_TYPE_ENABLED
from articles import views


app_name = 'articles'

type_prefix = '<str:type>/' if IS_ARTICLE_TYPE_ENABLED else ''


urlpatterns = [

    path(type_prefix, views.ArticleListView.as_view(), name='list'),

    path(type_prefix + '<str:slug>_<int:id>/',
         views.ArticleDetailView.as_view(), name='info')

]
